use async_trait::async_trait;
use octocrab;

use super::{PublishContext, ReleaseContext, ServerApi};
use crate::{
    config::{Api, Config},
    error::*,
};
pub struct Github {
    api: Api,
}

impl Github {
    pub async fn build(api: &Api) -> Result<Self> {
        Ok(Self {
            api: api.to_owned(),
        })
    }
}

#[async_trait]
impl ServerApi for Github {
    async fn init(&mut self, _cfg: &Config) -> Result<()> {
        Ok(())
    }
    async fn push_release(&mut self, rc: ReleaseContext) -> Result<()> {
        let response = octocrab::instance()
            .repos(rc.username, rc.repository)
            .releases()
            .create(&rc.tag)
            .target_commitish(&rc.commit_sig)
            .name(&rc.tag)
            .body(&rc.text)
            .draft(rc.draft)
            .prerelease(rc.prerelease)
            .send()
            .await
            .map_err(ServerApiError::from)?;
        Ok(())
    }
    async fn push_release_artifact(&mut self, _rc: ReleaseContext) -> Result<()> {
        todo!()
    }
    async fn push_pkg(&mut self, _pc: PublishContext) -> Result<()> {
        todo!()
    }
    fn get_cfg(&self) -> &Api {
        &self.api
    }
}
